@@ -34,25 +34,18 @@ import CADETMatch.util as util
 import CADETMatch.sub as sub
 
 
-def mirror(data):
-    data_max = numpy.max(data, 0)
+def mirror(data, meta_mask):
+    data_min = numpy.min(data, 0)
 
-    mirror_index = data_max <= 1.0
-    keep_index = data_max > 1.0
+    mirror_index = meta_mask
+    keep_index = numpy.logical_not(meta_mask)
 
-    data_min = data_max - data
     data_mask = numpy.ma.masked_equal(data_min, 0.0, copy=False)
     min_value = data_mask.min(axis=0)
 
-    data_mirror = numpy.zeros(data.shape)
+    data_mirror = numpy.copy(data)
 
-    data_mirror[:, mirror_index] = (
-        data_max[mirror_index]
-        + data_max[mirror_index]
-        - numpy.copy(data[:, mirror_index])
-        + min_value[mirror_index]
-    )
-    data_mirror[:, keep_index] = data[:, keep_index]
+    data_mirror[:, mirror_index] = -data_mirror[:, mirror_index]
     full_data = numpy.vstack([data_mirror, data])
 
     return full_data
@@ -63,7 +56,7 @@ def setupKDE(cache):
 
     mcmcDir = Path(cache.settings["resultsDirMCMC"])
 
-    scores_mirror = mirror(scores)
+    scores_mirror = mirror(scores, cache.meta_mask)
 
     scaler = getScaler(scores_mirror)
 
@@ -473,7 +466,7 @@ def generate_synthetic_error(cache):
         indexes = []
 
         for idx, (scores, simulations, outputs, errors) in enumerate(
-            cache.toolbox.map(synthetic_error_simulation, zip(json_path, errors_split))
+            cache.map_function(synthetic_error_simulation, zip(json_path, errors_split))
         ):
             if scores and simulations and outputs:
                 indexes.append(idx)
@@ -522,22 +515,16 @@ def generate_synthetic_error(cache):
 
         for output_name, output in outputs_all.items():
             kde_data.root[output_name] = numpy.array(output)[keep_idx, :]
-            kde_data.root.original[output_name] = numpy.array(output)[keep_idx, :]
 
         for time_name, time in times.items():
             kde_data.root["%s_time" % time_name] = time
-            kde_data.root.original["%s_time" % time_name] = time
 
         for name, experiment in errors_all.items():
             for error_name, error_value in experiment.items():
                 kde_data.root.errors[name][error_name] = error_value[keep_idx, :]
-                kde_data.root.original.errors[name][error_name] = error_value[
-                    keep_idx, :
-                ]
 
         for key, value in uv_store_all.items():
             kde_data.root.uv_store[key] = numpy.array(value)[keep_idx, :]
-            kde_data.root.original.uv_store[key] = numpy.array(value)[keep_idx, :]
 
         kde_data.save(lock=True)
 
