@@ -10,8 +10,8 @@ from pathlib import Path
 
 import h5py
 import numpy
+import functools
 from cadet import H5, Cadet
-from deap import base, creator, tools
 
 import CADETMatch.evo as evo
 import CADETMatch.gradFD as gradFD
@@ -25,7 +25,6 @@ def main(map_function):
     path = sys.argv[1]
     setup(cache, path, map_function)
     gradFD.setupTemplates(cache)
-    # grad.setupTemplates(cache)
     hof = evo.run(cache)
 
     multiprocessing.get_logger().info("altScoress %s", cache.altScores)
@@ -104,12 +103,18 @@ def setup(cache, json_path, map_function):
     print_version()
 
     cache.setup(json_path)
+    cache.map_function = map_function
+
+    cache.eval.evaluate = functools.partial(evo.fitness, json_path=json_path)
+    cache.eval.evaluate_final = functools.partial(evo.fitness_final, json_path=json_path)
+    cache.eval.evaluate_grad = functools.partial(gradFD.gradSearch, json_path=json_path)
+    cache.eval.evaluate_grad_fine = functools.partial(gradFD.gradSearchFine, json_path=json_path)
+    cache.eval.grad_search = gradFD.search
 
     createCSV(cache)
     createProgressCSV(cache)
     createErrorCSV(cache)
     setupTemplates(cache)
-    setupDeap(cache, map_function)
 
 
 def print_version():
@@ -120,24 +125,25 @@ def print_version():
     import importlib_metadata
 
     modules = [
-        ("joblib", "0.15.1"),
-        ("addict", "2.2.1"),
-        ("corner", "2.0.1"),
+        ('attrs', '21.2.0'),
+        ("joblib", "1.0.1"),
+        ("addict", "2.4.0"),
+        ("corner", "2.2.1"),
         ("emcee", "3.0.2"),
-        ("SALib", "0.0+nnone"),
-        ("deap", "1.3.1"),
-        ("psutil", "5.7.0"),
-        ("numpy", "1.18.5"),
-        ("openpyxl", "3.0.3"),
-        ("scipy", "1.5.0"),
-        ("matplotlib", "3.2.1"),
-        ("pandas", "1.0.5"),
-        ("h5py", "2.10.0"),
-        ("cadet", "0.8"),
-        ("seaborn", "0.10.1"),
-        ("scikit-learn", "0.23.1"),
-        ("jstyleson", "0.0.2"),
-        ('filelock', "3.0.12")
+        ("SALib", "1.3.11"),
+        ("psutil", "5.8.0"),
+        ("numpy", "1.21.1"),
+        ("openpyxl", "3.0.7"),
+        ("scipy", "1.7.0"),
+        ("matplotlib", "3.4.2"),
+        ("pandas", "1.3.0"),
+        ("h5py", "3.3.0"),
+        ("cadet-python", "0.11"),
+        ("seaborn", "0.11.1"),
+        ("scikit-learn", "0.24.2"),
+        ("jstyleson", "0.2.0"),
+        ('filelock', "3.0.12"),
+        ('pymoo', '0.4.2.2')
     ]
 
     for module, version_tested in modules:
@@ -341,24 +347,6 @@ def setupTemplates(cache):
         experiment["simulation_final"] = template_final
 
 
-def setupDeap(cache, map_function):
-    "setup the DEAP variables"
-    searchMethod = cache.settings.get("searchMethod", "NSGA3")
-    cache.toolbox = base.Toolbox()
-    cache.search[searchMethod].setupDEAP(
-        cache,
-        evo.fitness,
-        evo.fitness_final,
-        gradFD.gradSearch,
-        gradFD.search,
-        gradFD.gradSearchFine,
-        map_function,
-        creator,
-        base,
-        tools,
-    )
-
-
 def continue_mcmc(cache, map_function):
     if cache.continueMCMC:
         json_path = util.setupMCMC(cache)
