@@ -1,5 +1,6 @@
-import CADETMatch.util as util
 import numpy
+
+import CADETMatch.util as util
 from CADETMatch.abstract.transform import AbstractTransform
 
 
@@ -27,9 +28,16 @@ class VolumeAreaTransform(AbstractTransform):
 
     grad_transform = transform
 
+    def untransform_inputorder(self, seq):
+        volume = seq[0]
+        area = seq[1]
+        return [volume, area]
+
     def untransform(self, seq):
-        values = [seq[1], seq[0] / seq[1]]
-        headerValues = [values[0], values[1], values[0] * values[1]]
+        volume, area = self.untransform_inputorder(seq)
+        length = volume / area
+        values = [area, length]
+        headerValues = [area, length, volume]
         return values, headerValues
 
     def grad_untransform(self, seq):
@@ -51,7 +59,10 @@ class VolumeAreaTransform(AbstractTransform):
     def setSimulation(self, sim, seq, experiment):
         values, headerValues = self.untransform(seq)
 
-        if self.parameter.get("experiments", None) is None or experiment["name"] in self.parameter["experiments"]:
+        if (
+            self.parameter.get("experiments", None) is None
+            or experiment["name"] in self.parameter["experiments"]
+        ):
             area_location = self.parameter["area_location"]
             length_location = self.parameter["length_location"]
 
@@ -88,10 +99,10 @@ class VolumeAreaTransform(AbstractTransform):
         return headers
 
     def setBounds(self, parameter, lb, ub):
-        parameter["minVolume"] = lb[2]
-        parameter["maxVolume"] = ub[2]
-        parameter["minArea"] = lb[0]
-        parameter["maxArea"] = ub[0]
+        parameter["minVolume"] = lb[0]
+        parameter["maxVolume"] = ub[0]
+        parameter["minArea"] = lb[1]
+        parameter["maxArea"] = ub[1]
 
 
 class NormVolumeAreaTransform(VolumeAreaTransform):
@@ -163,7 +174,12 @@ class NormVolumeAreaTransform(VolumeAreaTransform):
         return [0.0, 0.0], [1.0, 1.0]
 
     def getGradBounds(self):
-        return [self.parameter["min"],], [self.parameter["max"],]
+        return [self.parameter["min"],], [
+            self.parameter["max"],
+        ]
 
 
-plugins = {"norm_volume_area": NormVolumeAreaTransform, "volume_area": VolumeAreaTransform}
+plugins = {
+    "norm_volume_area": NormVolumeAreaTransform,
+    "volume_area": VolumeAreaTransform,
+}
