@@ -19,10 +19,10 @@ import psutil
 import scipy.signal
 from addict import Dict
 from cadet import H5, Cadet
-from deap import tools
 
 import CADETMatch.calc_coeff as calc_coeff
 import CADETMatch.sub as sub
+import CADETMatch.pop as pop
 
 decim.getcontext().prec = 64
 __logBase10of2_decim = decim.Decimal(2).log10()
@@ -88,15 +88,17 @@ def generateIndividual(icls, size, imin, imax, cache):
 def initIndividual(icls, cache, content):
     return icls(content)
 
+def sobolPopulation(populationSize, populationDimension, lb, ub):
+    sobol = SALib.sample.sobol_sequence.sample(populationSize, populationDimension)
+    sobol = sobol * (ub - lb) + lb
+    return sobol
+
 
 def sobolGenerator(icls, cache, n):
     if n > 0:
         populationDimension = len(cache.MIN_VALUE)
         populationSize = n
-        sobol = SALib.sample.sobol_sequence.sample(populationSize, populationDimension)
-        lb = numpy.array(cache.MIN_VALUE)
-        ub = numpy.array(cache.MAX_VALUE)
-        sobol = sobol * (ub - lb) + lb
+        sobol = sobolPopulation(populationSize, populationDimension, numpy.array(cache.MIN_VALUE), numpy.array(cache.MAX_VALUE))
         data = numpy.apply_along_axis(list, 1, sobol)
         data = list(map(icls, data))
         return data
@@ -106,30 +108,19 @@ def sobolGenerator(icls, cache, n):
 
 def calcMetaScores(scores, cache):
     scores = numpy.array(scores)[cache.meta_mask]
-    if cache.allScoreNorm:
+    if cache.allScoreSSE and not cache.MultiObjectiveSSE:
+        sse = numpy.sum(numpy.array(scores))
+        return [sse, sse, sse]
+    else:
         prod_score = product_score(scores)
-        min_score = min(scores)
+        min_score = max(scores)
         mean_score = sum(scores) / len(scores)
-        human = [prod_score, min_score, mean_score]
-    elif cache.allScoreSSE:
-        if cache.MultiObjectiveSSE:
-            # scores = numpy.abs(scores)
-            prod_score = product_score(scores)
-            min_score = min(scores)
-            mean_score = sum(scores) / len(scores)
-            human = [prod_score, min_score, mean_score]
-        else:
-            sse = numpy.sum(numpy.array(scores))
-            human = [sse, sse, sse]
-    return human
+        return [prod_score, min_score, mean_score]
 
 
 def product_score(values):
     values = numpy.array(values)
-    if numpy.all(values >= 0.0):
-        return numpy.prod(values) ** (1.0 / len(values))
-    else:
-        return -numpy.prod(numpy.abs(values)) ** (1.0 / len(values))
+    return 1.0 - scipy.stats.gmean(1-values)
 
 
 def saveExperiments(save_name_base, settings, target, results, directory, file_pattern):
@@ -382,7 +373,6 @@ def runExperiment(
     if individual is not None:
         temp["individual"] = tuple(individual)
     temp["diff"] = []
-    temp["minimize"] = []
     temp["sim_time"] = []
     temp["sim_value"] = []
     temp["exp_value"] = []
@@ -399,8 +389,7 @@ def runExperiment(
                     sse_count,
                     sim_time,
                     sim_value,
-                    exp_value,
-                    minimize,
+                    exp_value
                 ) = cache.scores[featureType].run(
                     temp, target[experiment["name"]][featureName]
                 )
@@ -413,7 +402,6 @@ def runExperiment(
             temp["error"] += sse
             temp["error_count"] += sse_count
             temp["diff"].extend(diff)
-            temp["minimize"].extend(minimize)
             temp["sim_time"].append(sim_time)
             temp["sim_value"].append(sim_value)
             temp["exp_value"].append(exp_value)
@@ -837,13 +825,6 @@ def metaCSV(cache):
 def update_result_data(cache, ind, fit, result_data, results, meta_scores):
     if result_data is not None and results is not None:
         result_data["input"].append(tuple(ind))
-
-        if ind.strategy is not None:
-            result_data["strategy"].append(tuple(ind.strategy))
-        if ind.mean is not None:
-            result_data["mean"].append(tuple(ind.mean))
-        if ind.confidence is not None:
-            result_data["confidence"].append(tuple(ind.confidence))
         result_data["output"].append(tuple(fit))
         result_data["output_meta"].append(tuple(meta_scores))
 
@@ -896,7 +877,6 @@ def calcFitness(scores_orig, cache):
 
 
 def process_population(
-    toolbox,
     cache,
     population,
     fitnesses,
@@ -948,7 +928,7 @@ def process_population(
         ind.fitness.values = calcFitness(fit, cache)
         ind.csv_line = [time.ctime(), save_name_base] + csv_line
 
-        ind_meta = toolbox.individualMeta(ind)
+        ind_meta = pop.Individual(ind)
 
         ind_meta.fitness.values = meta_score
         ind_meta.csv_line = [time.ctime(), save_name_base] + csv_line
@@ -959,13 +939,11 @@ def process_population(
             csv_lines.append([time.ctime(), save_name_base] + csv_line)
 
             if halloffame is not None:
-                onFront, significant = pareto.updateParetoFront(halloffame, ind, cache)
+                onFront, significant = pareto.updateParetoFront(halloffame, ind)
                 if onFront and not cache.metaResultsOnly:
                     processResults(save_name_base, ind, cache, results)
 
-            onFrontMeta, significant_meta = pareto.updateParetoFront(
-                meta_hof, ind_meta, cache
-            )
+            onFrontMeta, significant_meta = pareto.updateParetoFront(meta_hof, ind_meta)
             if onFrontMeta:
                 meta_csv_lines.append([time.ctime(), save_name_base] + csv_line)
                 processResultsMeta(save_name_base, ind, cache, results)
@@ -974,11 +952,11 @@ def process_population(
             # None if used with algorithms like multistart or scoretest which don't need progress
             if progress_hof is not None:
                 onFrontProgress, significant_progress = pareto.updateParetoFront(
-                    copy.deepcopy(progress_hof), ind_meta, cache
+                    copy.deepcopy(progress_hof), ind_meta
                 )
                 if significant_progress:
                     made_progress = True
-                    pareto.updateParetoFront(progress_hof, ind_meta, cache)
+                    pareto.updateParetoFront(progress_hof, ind_meta)
 
     writer.writerows(csv_lines)
 
@@ -1018,7 +996,6 @@ def process_population(
 
 
 def eval_population(
-    toolbox,
     cache,
     invalid_ind,
     writer,
@@ -1030,8 +1007,7 @@ def eval_population(
     result_data=None,
 ):
     return eval_population_base(
-        toolbox.evaluate,
-        toolbox,
+        cache.eval.evaluate,
         cache,
         invalid_ind,
         writer,
@@ -1045,7 +1021,6 @@ def eval_population(
 
 
 def eval_population_final(
-    toolbox,
     cache,
     invalid_ind,
     writer,
@@ -1057,8 +1032,7 @@ def eval_population_final(
     result_data=None,
 ):
     return eval_population_base(
-        toolbox.evaluate_final,
-        toolbox,
+        cache.eval.evaluate_final,
         cache,
         invalid_ind,
         writer,
@@ -1073,7 +1047,6 @@ def eval_population_final(
 
 def eval_population_base(
     evaluate,
-    toolbox,
     cache,
     invalid_ind,
     writer,
@@ -1084,10 +1057,9 @@ def eval_population_base(
     generation,
     result_data=None,
 ):
-    fitnesses = toolbox.map(evaluate, map(list, invalid_ind))
+    fitnesses = cache.map_function(evaluate, map(list, invalid_ind))
 
     return process_population(
-        toolbox,
         cache,
         invalid_ind,
         fitnesses,
@@ -1181,10 +1153,7 @@ def cleanDir(dir, hof):
     exists = {str(path.name).split("_", 1)[0] for path in paths}
 
     # make set of allowed keys based on hall of hame
-    allowed = {
-        hashlib.md5(str(list(individual)).encode("utf-8", "ignore")).hexdigest()
-        for individual in hof.items
-    }
+    allowed = hof.hashes()
 
     # remove everything not in hall of fame
     remove = exists - allowed
@@ -1459,15 +1428,6 @@ def fractionate_sim(start_times, stop_times, components, simulation, unit):
     return fracs
 
 
-def translate_meta_min(score, cache):
-    temp = numpy.array(score)
-    if cache.allScoreSSE:
-        temp[:3] = -temp[:3]
-    else:
-        temp[:3] = 1 - temp[:3]
-
-    return temp
-
 def get_bins(array, axis=None):
     array = numpy.atleast_2d(array)
     shape = array.shape
